

#include <stdarg.h>

void pr(char *fmt, ... ){
        char buf[128]; // resulting string limited to 128 chars
        va_list args;
        va_start (args, fmt );
        vsnprintf(buf, 128, fmt, args);
        va_end (args);
        Serial.println(buf);

}

// Inputs
// AIN0 (pin 6) - potentiometer
// Analogue 1 - analogue in

// Outputs

// pins which drive the transmit transducer
// PWM on pins 9 and 10 (Timer 1)
#define OUT_PIN0 9
#define OUT_PIN1 10

 
#define AIN0 6
#define AIN1 7




#include <SerialCommand.h>

extern void __builtin_avr_delay_cycles(unsigned long __n);


#define BUFFER_SIZE 256


volatile unsigned int buffer[BUFFER_SIZE];
volatile byte tail = 0;




#define USPERIOD 195


// array with pairs of numebers 
// phase 0 count
// phase 1 count
// terminates with a 0 for the phase 0 count
// BUT the traling phase 1 count is loaded and has to be big enough
// for the interrupt to occur and turn off pwm - otherwise a trailing glitch
// is generated.


// pulse with phase change
unsigned int waveform[]=
{
// 0
USPERIOD,
USPERIOD,

 
 //1
USPERIOD,
USPERIOD,

 //2
USPERIOD,
USPERIOD,


 //3
USPERIOD,
USPERIOD,


 //4
USPERIOD,
USPERIOD,


 //5
USPERIOD*2,
USPERIOD,


 
  //6
USPERIOD,
USPERIOD,

  //7
USPERIOD,
USPERIOD,

  //8
USPERIOD,
USPERIOD,


  //9
USPERIOD,
USPERIOD,

 // stop
 0,
 USPERIOD, // this one needs to be large

};




unsigned int starttime;

void dumpdata(void)
{
 unsigned int p0;
   
 for(int i=1;i<80;i++)
 {
  if(i & 1) p0=(buffer[i]-starttime-USPERIOD) % (2*USPERIOD);
  else      p0=(buffer[i]-starttime) % (2*USPERIOD);
  
  pr("%u,%u",buffer[i]-starttime,p0);
 }

 Serial.flush();
}


//*** Process serial port commands

SerialCommand sCmd;



void cmdunrecognized(const char *command)
{
 Serial.println("What?"); 
}  


void cmdgetdata(void)
{
 dumpdata();
}





void loop() 
{
 TIMSK0 &= ~_BV(TOIE0); // disable timer0 overflow interrupt

 gopingpwm();
 capture();  

 TIMSK0 |= _BV(TOIE0); // enable timer0 overflow interrupt

 if(Serial.available()>0) 
 {
  sCmd.readSerial();
 }

 delay(100);
}


ISR(TIMER1_CAPT_vect) 
{
 if(tail>=BUFFER_SIZE) return;
 buffer[tail++] = ICR1;

 TCCR1B^=(1<<ICES1); // toggle sense of analogue comparator event
}



void capture(void)
{
 TCCR1A = 0x0;    // put timer1 in normal mode
 TCCR1B = 0x1;    // change prescaler to divide clock by 1

 // clear any pending capture or overflow interrupts
 TIFR1 = (1<<ICF1);

 // Enable input capture interrupt
 TIMSK1 |= (1<<ICIE1);
 
 pinMode(AIN0, INPUT);
 pinMode(AIN1, INPUT);
 DIDR1  |=  (1<< AIN0D);         // disable digital input buffer AIN0/1
 DIDR1  |=  (1<< AIN1D);

 ADCSRA=0; // ADC Control and Status Register A
 ADCSRB=(1<<ACME); // enable muliplxer
 ADMUX=0; // ADC Multiplexer Selection Register
 ACSR=(1<<ACIC); // Analog Comparator Input Capture by Timer/Counter 1: On

 ADMUX=1;                 // HCSR04 analogue output to analogue comparator

 TCCR1B |= (1<< ICES1);   //switch edge detector to detect the rise 

 unsigned int nowtime;
 tail=0;

 starttime=TCNT1;
 while(1)
 {
  nowtime=TCNT1;
  if((nowtime-starttime)>5000) break;
 }
}






unsigned int wavepointer;
volatile unsigned int wavecomplete;

ISR(TIMER1_OVF_vect) 
{
 if(waveform[wavepointer]==0)
 {
  TCCR1A = 0; // normal mode disconnect outputs
  TCCR1B = 0x1; // normal mode 
  wavecomplete=1;
  return;
 }
 
 ICR1=waveform[wavepointer]+waveform[wavepointer+1];
 wavepointer+=2;
 OCR1A=waveform[wavepointer+1];
 OCR1B=waveform[wavepointer+1];
}



void gopingpwm(void)
{
 pinMode(OUT_PIN0,OUTPUT);
 pinMode(OUT_PIN1,OUTPUT);

 digitalWrite(OUT_PIN0,1);
 
 TCCR1B = 0x0; // stop the clock
 TCNT1=0;
 TIFR1 = _BV(ICF1) | _BV(TOV1);  // clear any pending capture or overflow interrupts

 TCCR1A = _BV(COM1A1) | _BV(COM1B1) | _BV(COM1B0);    // put timer1 in normal mode

 TIMSK1 |= _BV(TOIE1);

 wavepointer=wavecomplete=0;
 ICR1=waveform[wavepointer]+waveform[wavepointer+1];
 OCR1A=waveform[wavepointer+1];
 OCR1B=waveform[wavepointer+1];
 wavepointer+=2;

 TCCR1A = _BV(COM1A1) | _BV(COM1B1) | _BV(COM1B0) | _BV(WGM11);

 OCR1A=waveform[wavepointer+1];
 OCR1B=waveform[wavepointer+1];
 
 TCCR1B = _BV(CS10) | _BV(WGM13) | _BV(WGM12); // start the clock
 
 while(!wavecomplete);
 TIMSK1 &= ~_BV(TOIE1); // clear overflow interrupt
}


 
void setup() 
{
 Serial.begin(9600);
 while (!Serial) { ; /* wait for serial port to connect. Needed for native USB */}

 sCmd.addCommand("get",cmdgetdata); 
}



